# Author:: Kazuhiro Yoshida.
# Copyright:: Copyright (c) 2005 Kazuhiro Yoshida. All rights reserved.
# License:: Ruby license.

require 'webrick'
require 'test/unit/ui/testrunnermediator'
require 'test/unit/ui/testrunnerutilities'

module Test
  module Unit
    module UI
      module Webrick

        class TestRunner

          extend TestRunnerUtilities
          include WEBrick::HTMLUtils

          # Creates a new TestRunner for running the passed
          # suite.
          def initialize(suite, output_level = NORMAL)
            @encoding = 
              case $KCODE
              when "SJIS"
                "Shift_JIS"
              when "EUC"
                "EUC-JP"
              else
                "UTF-8"
              end
            if (suite.respond_to?(:suite))
              @suite = suite.suite
            else
              @suite = suite
            end

            @t = nil
            @browser = nil
            @result = nil
          end
  
          # Begins the test run.
          def start
            setup_ui
            setup_mediator
            attach_to_mediator
            start_ui
            sleep(0.2)
            @result
          end

          def setup_mediator
            @mediator = TestRunnerMediator.new(@suite)
          end
          
          def attach_to_mediator
            @mediator.add_listener(TestRunnerMediator::RESET, &method(:reset_ui))
            @mediator.add_listener(TestResult::FAULT, &method(:add_fault))
            @mediator.add_listener(TestResult::CHANGED, &method(:result_changed))
            @mediator.add_listener(TestRunnerMediator::STARTED, &method(:started))
            @mediator.add_listener(TestCase::STARTED, &method(:test_started))
            @mediator.add_listener(TestRunnerMediator::FINISHED, &method(:finished))
          end
  
          def start_ui
            @mediator.run_suite
          end
          
          def stop
            Thread.kill @t if @t
            @browser.close if @browser
          end
          
          def do_command(command, options)
            xml = %[<?xml version="1.0" encoding="#{@encoding}"?>]
            xml << %[<document>]
            xml << %[<command>#{command}</command>]
            options.each {|key, value|
              xml << %[<#{key}>#{escape value.to_s}</#{key}>]
            }
            xml << %[</document>]
            timeout(@timeout) do
              result = nil
              @t = Thread.new do
                # puts "about to push #{xml} onto the out queue"
                @out_queue.push(xml)
                if command != "Finished"
                  # puts "Waiting for next reply/request"
                  result = @in_queue.pop
                end
              end
              @t.join
              case result
              when nil, "OK", "PASSED"
                #
              when "RESTART"
                @mediator.run_suite
              else
                raise SeleniumCommandError, result
              end
              result
            end
          end
          
          def reset_ui(count)
            do_command "RESET", :total_count => count
          end
          
          def add_fault(fault)
            do_command "FAULT", :message => fault.short_display
          end
          
          def result_changed(result)
            do_command "CHANGED", 
              :run_count => result.run_count, 
              :assertion_count => result.assertion_count, 
              :failure_count => result.failure_count, 
              :error_count => result.error_count
          end

          def started(result)
            @result = result
            output_status("Started...")
          end
          
          def test_started(test_name)
            output_status("Running #{test_name}...")
          end
          
          def finished(elapsed_time)
            output_status("Finished in #{elapsed_time} seconds")
            do_command "Finished", :elapsed_time => elapsed_time
          end
          
          def output_status(string)
            do_command "STATUS", :message => string
          end
  
          def setup_ui
            @timeout = 1000
            @in_queue = Queue.new
            @out_queue = Queue.new
            Selenium::WebrickCommandProcessor.new(@in_queue, @out_queue)
            @browser = Selenium::WindowsIEBrowserLauncher.new
            @browser.launch("http://localhost:7896/test_runner.html")
            trap("INT") { stop }
          end
        end

      end
    end
  end
end

# = module Selenium
# original archive:
#   selenium-0.2-ruby.zip
#   http://selenium.thoughtworks.com/
# original lisence:
# Copyright 2004 ThoughtWorks, Inc
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

module Selenium

  class WebrickCommandProcessor
    def initialize(in_queue, out_queue)
      server = WEBrick::HTTPServer.new(
        :ServerType => Thread,
        :Port => 7896,
        :Logger => WEBrick::BasicLog.new(nil, WEBrick::BasicLog::WARN), #comment out to enable server logging
        :AccessLog => {} #comment out to enable access logging
      )
      # AUT
      server.mount("/", Selenium::NonCachingFileHandler, File.dirname(__FILE__) + "/html")

      # Selenium's static files and dynamic handler
      server.mount("/test", Selenium::NonCachingFileHandler, File.dirname(__FILE__) + "/test")
      server.mount_proc("/test/driver") do |req, res|
        res["Cache-control"] = "no-cache"
        res["Pragma"] = "no-cache"
        res["Expires"] = "-1"

        # Only handle "GET" for now
        if "GET" == req.request_method
          command_result = req.query['commandResult']
          selenium_start = req.query['seleniumStart']
          # puts "--> #{command_result} , #{selenium_start}"
          in_queue.push(command_result) unless selenium_start
          get_reply = out_queue.pop
          # puts "--> #{get_reply}"
          res.content_type = 'text/xml'
          res.body = get_reply
        end
      end
      trap("INT") { server.shutdown }
      server.start
    end

  end

  class NonCachingFileHandler < WEBrick::HTTPServlet::FileHandler
    def service(req, res)
      res["Cache-control"] = "no-cache"
      res["Pragma"] = "no-cache"
      res["Expires"] = "-1"
      super
    end
  end

  # Will create and then close an instance of IE
  class WindowsIEBrowserLauncher
    def initialize()
      require 'win32ole'
      @ie = nil
    end
    def launch(url)
      @ie = WIN32OLE.new('InternetExplorer.Application') if @ie == nil
      show
      @ie.navigate(url)
    end
    def close()
      sleep 0.2 # a hack, we should be waiting for the testcomplete command to be processed
      @ie.quit if @ie
      @ie = nil
    end
    def show
      @ie.visible = true
    end
    def hide
      @ie.visible = false
    end
  end

  module SpecifiedPathBrowserLauncher
    @@executable = nil
    def initialize(executable = @@executable)
      @executable = executable || raise("No executable specified")
    end
  end

  class UnixSpecifiedPathBrowserLauncher
    include SpecifiedPathBrowserLauncher
    def launch(url)
      @pid = Process.fork{exec "#{@executable} #{url}"}
    end
    def close
      Process.kill('SIGKILL', @pid)
    end
  end

  class WindowsSpecifiedPathBrowserLauncher
    include SpecifiedPathBrowserLauncher
    def launch(url)
      @thread = Thread.new{system("#{@executable} #{url}")}
    end
    def close; end # need to find a way
  end

  class WindowsSpecifiedPathBrowserLauncher2
    include SpecifiedPathBrowserLauncher
    def launch(url)
      system("start #{@executable} #{url}")\
    end
    def close; end # need to find a way
  end

  # Will use existing browser if available; won't close it regardless
  class WindowsDefaultBrowserLauncher
    def launch(url)
      system('cmd /c start ' + url)
    end
    def close; end
  end

end

class SeleniumCommandError < RuntimeError
end

if __FILE__ == $0
  Test::Unit::UI::Webrick::TestRunner.start_command_line_test
end
